#include <Arduino.h>

#include <Ethernet.h>
#include <ArduinoHA.h>
#include <JC_Button.h>

#define BROKER_ADDR     IPAddress(192,168,0,140)
#define BROKER_USERNAME     "mqtt" // replace with your credentials
#define BROKER_PASSWORD     "1234"

byte mac[] = {0x00, 0x10, 0xFA, 0x6E, 0x38, 0x4B}; // sen unique in ethernet segment

EthernetClient client;
HADevice device(mac, sizeof(mac));
HAMqtt mqtt(client, device);
HATriggers triggers;
HASensor temp("temp");


typedef struct {
  int           switchPin;
  const char *  switchDescription;
  HASwitch      switchObject;
  const char *  switchIcon;
} StructSwitch;

StructSwitch mySwitches[] = {
  {18, "Wiatrolap",         HASwitch("ledA", false), "mdi:lightbulb" },
  {17, "Pralnia",           HASwitch("ledB", false), "mdi:LightbulbVariantOutline" },
  {16, "Dobudówka",         HASwitch("ledC", false), "mdi:mdiLightbulbMultiple" },
  {15, "Kibelek",           HASwitch("ledD", false), "mdi:LightbulbMultiple" },
  {14, "Lazienka1",         HASwitch("ledE", false), "mdi:flash" },
  {13, "Lazienka2",         HASwitch("ledF", false), "mdi:flash" },
  {12, "Taras",             HASwitch("ledG", false), "mdi:flash" },
  {11, "Piec",              HASwitch("ledH", false), "mdi:heater" },

};

typedef struct {
  int           adruinoPin;
  const char *  buttonName;
  Button        buttonObject;
} StructButton;

StructButton myButtons[] = {
    {A0,"gab01",Button(A0)}, 
    {A1,"gab02",Button(A1)}, 
    {A2,"syp01",Button(A2)}, 
    {A3,"syp02",Button(A3)},
    {A4,"syp03",Button(A4)},
    {A5,"syp04",Button(A5)},
    {A6,"syp05",Button(A6)},
    {A7,"syp06",Button(A7)},
    {A8,"syp07",Button(A8)},
    {A9,"syp08",Button(A9)},
    {A10,"kor01",Button(A10)}, 
    {A11,"kor02",Button(A11)}, 
    {A12,"kor03",Button(A12)}, 
    {A13,"kor04",Button(A13)}, 
    {A14,"kib01",Button(A14)}, 
    {A15,"kib02",Button(A15)}, 
    {49,"laz01",Button(49)}, 
    {48,"laz02",Button(48)}, 
    {47,"wia01",Button(47)}, 
    {46,"kch01",Button(46)},
    {45,"kch02",Button(45)},
    {44,"kch03",Button(44)},
    {43,"kch04",Button(43)},
    {42,"kch05",Button(42)},
    {41,"kch06",Button(41)},
    {40,"kch07",Button(40)},
    {39,"kch08",Button(39)},
    {38,"kch09",Button(38)},
    {37,"kch10",Button(37)},
    {36,"sal01",Button(36)}, 
    {35,"sal02",Button(35)}, 
    {34,"sal03",Button(34)}, 
    {33,"sal04",Button(33)}, 
    {32,"kog01",Button(32)}, 
    {31,"kog02",Button(31)}, 
    {30,"lag01",Button(30)}, 
    {29,"lag02",Button(29)}, 
    {28,"ant01",Button(28)}, 
    {27,"ant02",Button(27)}, 
    {26,"ant03",Button(26)}, 
    {25,"mch01",Button(25)}, 
    {24,"mch02",Button(24)}, 
    {23,"sta01",Button(23)}, 
    {22,"sta02",Button(22)}, 
    {21,"sta03",Button(21)}, 
    //{20,"tar01",Button(20)}, 
    {19,"tar02",Button(19)},
};

const int numberOfMyButtons = sizeof(myButtons) / sizeof(StructButton);

// Button state:holding
bool holdingBtn = false;

const int numberOfSwitches = sizeof(mySwitches) / sizeof(StructSwitch);


void onBeforeSwitchStateChanged(bool state, HASwitch* s)
{
    // this callback will be called before publishing new state to HA
    // in some cases there may be delay before onStateChanged is called due to network latency
}

void onSwitchStateChanged(bool state, HASwitch* s)
{
        for (int i = 0; i < numberOfSwitches; i++) {
          if (mySwitches[i].switchDescription == s->getName()) {

            digitalWrite(mySwitches[i].switchPin, (state ? HIGH : LOW ));

          };
        };
}

void triggerButtonChange(Button* s, const char *ButtonName) {

    s->read();


    if (s->pressedFor(1000) && !holdingBtn) {
        Serial.println(ButtonName);
        triggers.trigger("button_long_press", ButtonName);        
        holdingBtn = true;
    } else if (s->wasReleased()) {
        if (holdingBtn) {
            holdingBtn = false;
        } else {
            triggers.trigger("button_short_press", ButtonName);
        }
    }

}

void setup() {
  // you don't need to verify return status
  Ethernet.begin(mac);

  // set device's details (optional)
  device.setName("Arduino_Lights");
  device.setSoftwareVersion("1.0.8");
  device.setModel("ATmega2560");

  // switche
  for (int i = 0; i < numberOfSwitches; i++) {

    pinMode(mySwitches[i].switchPin, OUTPUT);

    mySwitches[i].switchObject.setIcon(mySwitches[i].switchIcon);
    mySwitches[i].switchObject.setName(mySwitches[i].switchDescription);

    mySwitches[i].switchObject.onBeforeStateChanged(onBeforeSwitchStateChanged); //optional
    mySwitches[i].switchObject.onStateChanged(onSwitchStateChanged);
  };

  //buttony
  for (int i = 0; i < numberOfMyButtons; i ++) {
    // setup triggers
    triggers.add("button_short_press", myButtons[i].buttonName);
    triggers.add("button_long_press", myButtons[i].buttonName);

    myButtons[i].buttonObject.begin();

  }



  mqtt.begin(BROKER_ADDR,BROKER_USERNAME,BROKER_PASSWORD);
}

void loop() {
    Ethernet.maintain();
    mqtt.loop();


    for (int i = 0; i < numberOfMyButtons; i++) {
       // trigger mqtt after button press
       triggerButtonChange(&myButtons[i].buttonObject,myButtons[i].buttonName);
    };


}
